/*! DEFINE: FRAMEWORK NAMESPACES **/
$framework = $(framework = ($$ = window.$$ || window.framework || {}));

/*! POLYFILL: ARRAY.UNSHIFT() */
/*! Unshift Polyfill | @link bennadel.com/blog/2300-creating-a-javascript-shim-for-array-unshift-method.htm | @author Ben Nadel */
if(!Array.prototype.hasOwnProperty("unshift")){Array.prototype.unshift=function(){Array.prototype.splice.call(arguments,0,0,0,0);Array.prototype.splice.apply(this,arguments);return(this.length);};};

/*! POLYFILL: STRING.INCLUDES() */
/*! Includes Polyfill | @link developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/includes | @author Mozilla */
if (!String.prototype.includes){String.prototype.includes=function(search, start){'use strict'; if (typeof start !=='number'){start=0;}if (start + search.length > this.length){return false;}else{return this.indexOf(search, start) !==-1;}};}

/*! POLYFILL: STRING.TRIM() */
/*! Trim Polyfill | @link developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/Trim | @author Mozilla */
if(!String.prototype.hasOwnProperty("trim")){String.prototype.trim=function(){return this.replace(/^[\s\uFEFF]+|[\s\uFEFF]+$/g,'');};};

/*! FUNCTION: ARRAY.KEYSORT(); **/
Array.prototype.keySort = function(key, desc){
  this.sort(function(a, b) {
    //return desc ? ~~(a[key] < b[key]) : ~~(a[key] > b[key]);
    var result = desc ? (a[key] < b[key]) : (a[key] > b[key]);
    return result ? 1 : -1;
  });
  return this;
}
/*
array.keySort('key');
array.keySort('key', true);
*/

/*! FUNCTION: NUMBER.FORMATCURRENCY(); **/
Number.prototype.formatCurrency = function(c, d, t){
  var n = this, c = isNaN(c = Math.abs(c)) ? 2 : c, d = d == undefined ? "." : d, t = t == undefined ? "," : t, s = n < 0 ? "-" : "", i = parseInt(n = Math.abs(+n || 0).toFixed(c)) + "", j = (j = i.length) > 3 ? j % 3 : 0;
  return s + (j ? i.substr(0, j) + t : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + t) + (c ? d + Math.abs(n - i).toFixed(c).slice(2) : "");
}
/*
(123456789.12345).formatCurrency();
(1234.56789).formatCurrency(2,'.',',');
*/

/*! FUNCTION: STRING.FORMATCURRENCY(); **/
String.prototype.formatCurrency = function(){
  var value = parseFloat(this.toString().replace(/[^0-9\.]/g, ''));
  return Number.prototype.formatCurrency.apply(value, arguments);
}
/*
"$123456789.12345".formatCurrency();
"$1234.56789".formatCurrency(2,'.',',');
*/

/*! FUNCTION: NUMBER.ROUNDTONEAREST(); **/
Number.prototype.roundToNearest = function(step, method){
  if (typeof step === 'undefined') {
    step = 1;
  }
  if (typeof method === 'undefined') {
    method = 'round';
  }
  return Math[method](this / step) * step;
}
/*
(1.5).roundToNearest();
(1.5).roundToNearest(5);
(1.5).roundToNearest(5, 'floor');
(1.5).roundToNearest(5, 'round');
(1.5).roundToNearest(5, 'ceil');
*/

/*! FUNCTION: 'STRING'.TOTITLECASE() **/
String.prototype.toTitleCase = function(){
  return this.replace(/\w\S*/g, function(txt){return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();});
}
/*
'sOME sTRING'.toTitleCase(); // 'Some String'
*/

/*! FUNCTION: 'STRING'.TOCAMELCASE() **/
String.prototype.toCamelCase = function() {
  return this.replace(/^([A-Z])|[\s-_](\w)/g, function(match, p1, p2, offset) {
    if (p2) {
      return p2.toUpperCase();
    } else {
      return p1.toLowerCase();
    }
  });
}
/*
'Some-String-2'.toCamelCase(); // 'someString2'
*/

/*! FUNCTION: 'STRING'.DECAMELCASE() **/
String.prototype.deCamelCase = function(sep){
  if (typeof sep == 'undefined'){var sep = '-'}
  return this.replace(/([A-Z0-9]+)/g, ' $1').replace(/([A-Z][a-z])/g, ' $1').deSpace(sep).toLowerCase();
}
/*
'someString2'.deCamelCase(); // 'some-string-2'
'someString2'.deCamelCase(' '); // 'some string 2'
'someString2'.deCamelCase('_'); // 'some_string_2'
*/

/*! FUNCTION: 'STRING'.TOVARIABLE() **/
String.prototype.toVariable = function(){
  return this.deSpecialize().toCamelCase();
}
/*
'some String2'.toVariable(); // 'someString2'
'some!String2'.toVariable(); // 'someString2'
'some_String2'.toVariable(); // 'someString2'
*/

/*! FUNCTION: 'STRING'.DESPACE() **/
String.prototype.deSpace = function(sep){
  if (typeof sep == 'undefined'){var sep = ''}
  return this.trim().replace(/\s+/g, sep);
}
/*
'some String2'.deSpace('_'); // 'some_String2'
'some!String2'.deSpace(); // 'some!String2'
'some_String2'.deSpace(); // 'some_String2'
*/

/*! FUNCTION: 'STRING'.DESPECIALIZE() **/
String.prototype.deSpecialize = function(sep){
  if (typeof sep == 'undefined'){var sep = ' '}
  return this.replace(/[^\w\s]/gi, sep);
}
/*
'some!String2'.deSpecialize(); // 'some String2'
'some_String2'.deSpecialize(); // 'some_String2'
'some!String2'.deSpecialize('_'); // 'some_String2'
*/

/*! FUNCTION: 'STRING'.SIMPLIFY() **/
String.prototype.simplify = function(sep,special){
  var str = this;
  if (typeof sep == 'undefined'){var sep = ''}
  /*
  if (typeof special != 'undefined' && special){str = str.replace(/([\/\,\!\\\^\$\{\}\[\]\(\)\.\*\+\?\|\<\>\-\&])/g, ' ')}
  */
  if (typeof special != 'undefined' && special){str = str.deSpecialize(' ')}
  return str.deCamelCase(sep);
}
/*
'someString2'.simplify(); // 'somestring2'
'someString2'.simplify('-'); // 'some-string-2'
'someString2'.simplify(' '); // 'some string 2'
'someString&2'.simplify('-', true); // 'some-string-2'
*/

/*! FUNCTION: 'STRING'.REPLACEALL() **/
String.prototype.replaceAll = function(str1, str2, ignore){
  return this.replace(new RegExp(str1.replace(/([\/\,\!\\\^\$\{\}\[\]\(\)\.\*\+\?\|\<\>\-\&])/g,"\\$&"),(ignore?"gi":"g")),(typeof(str2)=="string")?str2.replace(/\$/g,"$$$$"):str2);
}
/*
'!'.replaceAll('!', 'xyz'); // 'xyz'
'!'.replaceAll('', 'xyz'); // 'xyz!xyz'
'aAa'.replaceAll('a', 'b', true); // 'bbb'
'Hello???'.replaceAll('?', '!'); // 'Hello!!!'
*/

/*! FUNCTION: 'STRING'.DECODEHTML() **/
String.prototype.decodeHTML = function(){
  return $('<div/>').html(this).text();
}
/*
'&amp;&quot;&#39;&lt;&gt;'.decodeHTML(); // '&"'<>'
*/

/*! FUNCTION: 'STRING'.ENCODEHTML() **/
String.prototype.encodeHTML = function(){
  return String(this).replace(/&/g, '&amp;').replace(/"/g, '&quot;').replace(/'/g, '&#39;').replace(/</g, '&lt;').replace(/>/g, '&gt;');
}
/*
'&"\'<>'.encodeHTML(); // '&amp;&quot;&#39;&lt;&gt;'
*/

/*! FUNCTION: $$.ADDTOCART(); **/
$$.addToCart = function(product, category, quantity, callbacks){
  // SETUP
  if (typeof product === 'string' || typeof product === 'number') {
    var item = product.toString();
    item = item.indexOf('i_') > -1 ? item : 'i_'+item.replace('_',' ');
    product = {};
    product.partNumber = item;
  } else
  if (typeof product === 'undefined' || !product.partNumber) {
    console.error('Invalid Product!');
    return false;
  }
  if (typeof category === 'undefined' || !category) {
    /*
    var category = 'Search|On Site Search';
    */
    var category = $$.properties.siteName;
  }
  if (typeof quantity === 'undefined' || !quantity) {
    var quantity = 1;
  }
  if (typeof callbacks === 'undefined') {
    var callbacks = {};
  }
  /*
  if (typeof onOpen === 'undefined') {
    var onOpen = false;
  }
  if (typeof onClose === 'undefined') {
    var onClose = false;
  }
  */
  /*
  // TODO (DETECT MOBILE)
  var mobile = false;
  */
  // AJAX
  $.ajax({
    type: 'POST',
    url: /* mobile ? '/checkout/cart' : */ '/OrderItemUpdate',
    data: /* mobile ? {
      id: product.mfPartNumber,
      skuid: product.prodId,
      categoryId: category,
      qty: quantity
    } : */ {
      storeId: $$.properties.storeId,
      catalogId: $$.properties.catalogId,
      URL: 'SuccessView',
      partNumber_0: product.partNumber,
      catEntryId_0: product.prodId,
      quantity_0: quantity,
      comment_0: category,
      pAttr_0: 'CMAttributes',
      pAttrVal_0: '-_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_-quickadd-_--_--_--_--_--_--_-'+$$.properties.pageId+'-_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_-'
    },
    complete: function(xhr, status){
      var success = status === 'success' ? true : false;
      $$.renderTemplate({
        template: 'cart_confirm',
        target: 'body',
        method: 'append',
        product: product,
        category: category,
        quantity: quantity,
        success: success,
        callback: function(config){
          var $container = config.module;
          var $checkout = $container.find('.checkout > .btn');
          // HANDLE INTERACTION
          $checkout.on('click', function(event){
            // GOTO SHOPPING CART
            window.location.href = '/OrderCalculate?URL=/ShoppingCartView';
          });
          // INITIALIZE MODAL
          $container.modal().on('hidden.bs.modal', function(){
            $(this).remove();
            // CLOSE CALLBACK
            callbacks.onClose && callbacks.onClose(product, category, quantity, success);
          });
          // OPEN CALLBACK
          callbacks.onOpen && callbacks.onOpen(product, category, quantity, success);
          // UPDATE CART
          $$.fetchFeed({
            feed: '/OrderCalculate?URL=/ShoppingCartView',
            type: 'text',
            callback: $$.refreshCart
          });
          // UPDATE CART
          $.ajax({
            type: 'POST',
            url: '/OrderCalculate?URL=/SuccessView',
            data: {
              storeId: $$.properties.storeId,
              catalogId: $$.properties.catalogId,
              langId: -1,
              updatePrices: 1,
              calculationUsageId: -1,
              calculationUsageId: -2,
              calculationUsageId: -7,
              orderId: '.',
              addToCartDiv: 1
            }
          });
        }
      });
      // TAGGING
      if (success) {
        var cmAttributes = '-_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_-quickadd-_--_--_--_--_--_--_-'+$$.properties.pageId+'-_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_-';
        //cmCreateShopAction5Tag(product.partNumber.replace(/^i_/, ''), product.prodName, quantity, product.contractPrice, $$.properties.siteName, cmAttributes);
        cmCreateShopAction5Tag(product.mfPartNumber, product.prodName, quantity, product.contractPrice, $$.properties.siteName, cmAttributes);
        //console.log(product.partNumber.replace(/^i_/, ''), product.prodName, quantity, product.contractPrice, $$.properties.siteName, cmAttributes);
        cmDisplayShops();
        $(document).trigger('quickadd', [{mfPartnumber: product.mfPartNumber, name: product.prodName, quantity: quantity, price: product.contractPrice, categoryId: category, storeId: $$.properties.storeId, currencyCode: 'USD', overrideCategoryID: null, cmAttributes: cmAttributes}]);
      }
    }
  });
}
/*
$$.addToCart('35398_WHI');
*/

/*! FUNCTION: $$.REFRESHCART(); **/
$$.refreshCart = function(){
  var done = function(response) {
    if (response.items && response.items.length > 0) {
      var text = 'Cart <span>('+response.items.length+')</span>';
    } else {
      var text = 'Cart (0)';
    }
    $('header').find('.links, .utilities').find('.cart > a').html(text);
  }
  $$.fetchFeed({
    feed: '/MiniCartView',
    callback: done,
    error: done
  });
}
/*
$$.refreshCart();
*/

/*! FUNCTION: $$.APPLYSOURCECODE(); **/
$$.applySourceCode = function(code){
  $body.append('<iframe class="hidden" src="/MobileTemplateView?contentKey=ApplySourceCode&SourceCode='+code+'"></iframe>');
}
/*
$$.applySourceCode('XXW12345');
*/

/*! FUNCTION: HIJACKING SETSOURCECODE; **/
$$.setSourceCode = function (SourceCode) {
  url = '/PromoCodeAtCheckout';
  SourceCodeData = {
    'promoCode': SourceCode,
    'isFromBanner': 'true'
  };
  $.ajax({
   url: url,
   type: 'post',
   data: SourceCodeData
  }).done(function(data) {
    var response = JSON.parse(data);
    var isValid = response.isValid;
    if (isValid == 'true') {
      var sourceCode = response.sourceCode;  
      jQuery('#sourceCode').text('Offer Code: ' + sourceCode);
      if(breadcrumb.area == 'shopping-cart-area' || breadcrumb.area == 'check-out-area') {
        window.location.reload();
      } else {
        $$.goModal({
          title: 'Offer Code Applied',
          html: '<p class="margin">Offer Code <strong>'+sourceCode+'</strong> has been applied to your cart.</p>',
          width: 600,
          initialHeight: 100,
          scrolling: false
        });
      }
    } else { 
      $$.goModal({
        title: 'Error',
        html: '<p class="margin">The Offer Code <strong>'+sourceCode+'</strong> was not applied. Please enter a valid offer code.</p>',
        width: 600,
        initialHeight: 100,
        scrolling: false
      });
    }
  }).fail(function() {
    $$.goModal({
      title: 'Error',
      html: '<p class="margin">The Offer Code <strong>'+sourceCode+'</strong> was not applied. Please enter a valid offer code.</p>',
      width: 600,
      initialHeight: 100,
      scrolling: false
    });
  });
}
/*
$$.setSourceCode('XXW12345');
*/

/*! FUNCTION: $$.ASSOCIATEOBJ() **/
$$.associateObj = function(keys, values){
  var obj = {};
  for (var x = 0; i < keys.length; i++){
    obj[keys[i]] = values[i];
  }
  return obj;
};
/*
$$.breakpoints.keys = ['Mobile', 'Tablet', 'Desktop'];
$$.breakpoints.values = [1, 768, 1025];

$$.associateObj($$.breakpoints.keys, $$.breakpoints.values); // {Mobile: 1, Tablet: 768, Desktop: 1025}
*/

/*! FUNCTION: $$.UNASSOCIATEOBJ() **/
$$.unassociateObj = function(obj, type){
  var index = [];
  var keys = [];
  var values = [];
  for (var item in obj){
    if (typeof obj[item] === 'object'){
      index.push(item);
      for (var key in obj[item]){
        keys.push(key);
        values.push(obj[item][key]);
      }
    } else {
      keys.push(item);
      values.push(obj[item]);
    }
  }
  return (typeof type === 'undefined') ? (index.length > 0 ? {index:index, keys:keys, values:values} : {keys:keys, values:values}) : ((type === 'index') ? index : (type === 'keys') ? keys : values);
};
/*
$$.breakpoints = {Mobile: {xs: 0}, Tablet: {sm: 768}, Medium: {md: 992}, Large: {lg: 1200}};

$$.unassociateObj($$.breakpoints); // {index: ['Mobile', 'Tablet', 'Medium', 'Large'], keys: ['xs', 'sm', 'md', 'lg'], values: [1, 768, 992, 1025]}
$$.unassociateObj($$.breakpoints, 'index'); // ['Mobile', 'Tablet', 'Medium', 'Large']

$$.breakpoints = {Mobile: 1, Tablet: 768, Desktop: 1025};

$$.unassociateObj($$.breakpoints); // {keys: ['Mobile', 'Tablet', 'Desktop'], values: [1, 768, 1025]}
$$.unassociateObj($$.breakpoints, 'keys'); // ['Mobile', 'Tablet', 'Desktop']
$$.unassociateObj($$.breakpoints, 'values'); // [1, 768, 1025]
*/

/*! FUNCTION: $$.FILTERARRAY() **/
$$.filterArray = function(array, filter, key){
  array = (typeof array == 'object') ? array : [array];
  filter = (typeof filter == 'object') ? filter : [filter];
  if (filter == '*' || filter.length == 0){
    return array;
  } else {
    var output = [];
    $.each(array, function(x1, y1){
      // FILTER IS ASSOCIATIVE (DOESNT DE-DUPE!)
      if (typeof filter[0] == 'object' && filter[0].length == undefined) {
        $.each(filter, function(x2, y2){
          if ((y1[key] || y1) == (y2[key] || y2)) {
            output.push(array[x1]);
          }
        });
      // FILTER IS NOT ASSOCIATIVE (DOES DE-DUPE!)
      } else {
        if ($.inArray(y1[key] || y1, filter) > -1){
          output.push(array[x1]);
        }
      }
    });
    return output;
  }
}
/*
$$.filterArray(1,'*'); // [1]
$$.filterArray(1,[1,3,5]); // [1]
$$.filterArray([1,2,3,4,5],1); // [1]
$$.filterArray([1,2,3,4,5],[1,3,5]); // [1,3,5]
$$.filterArray([1,3,5],[{0:1},{0:2},{0:3},{0:4},{0:5}],0); // [1,3,5]
$$.filterArray([{0:1},{0:2},{0:3},{0:4},{0:5}],[1,3,5],0); // [{0:1},{0:3},{0:5}]
$$.filterArray([{0:1},{0:2},{0:3},{0:4},{0:5}],[{0:1},{0:3},{0:5}],0); // [{0:1},{0:3},{0:5}]
*/

/*! FUNCTION: $$.RECURSIVEARRAY() **/
$$.recursiveArray = function(array, callback){
  for (var x = 0; x < array.length; x++){
    callback(array[x],x);
  }
}
/*
$$.recursiveArray(['A','B','C'],function(data){console.log(data)}); // A // B // C
$$.recursiveArray([{x:1},{x:2}],function(data){console.log(data.x)}); // 1 // 2
$$.recursiveArray(['A','B','C'],function(y,x){console.log(x)}); // 0 // 1 // 2
*/

/*! FUNCTION: $$.SEARCHSTRING() **/
$$.searchString = function(data){
  for (var x = 0; x < data.length; x++){
    if (typeof data[x].search !== 'object'){
      data[x].search = new RegExp(data[x].search, 'i');
    }
    if (data[x].string && data[x].search.test(data[x].string)){
      return data[x].match;
    }
    else if (data[x].prop){
      return data[x].match;
    }
  }
  return 'Unknown';
}
/*
$$.searchString([{string: navigator.platform, search: 'Win', match: 'Windows'}]); // 'Windows'
*/

/*! FUNCTION: $$.BUILDCONFIG(); **/
$$.buildConfig = function(config, callback){
  // SETUP
  if (typeof config === 'undefined' || !config.product) {
    return false;
  }
  if (typeof callback === 'function') {
    config.callback = callback;
  }
  // PRODUCT MODEL
  var productModel = function(){
    // DEFINE VARIABLES
    var current = config.product;
    var last = config.product.definingAttributes.keySort('optionSequence').length-1;
    // STEP THRU EACH ENTITLED ITEM
    $.each(config.product.entitledItems, function(itemKey, item) {
      // ADD ITEM PROPERTIES
      item.prodName = config.product.prodName;
      item.mfPartNumber = config.product.mfPartNumber;
      // CAN THIS BE DONE MORE EFFICIENTLY?
      $.each(item.definingAttributes, function(optionKey, option) {
        $.each(config.product.definingAttributes, function(attributeKey, attribute) {
          if (option.optionItemKey === attribute.optionItemKey) {
            option.optionSequence = attribute.optionSequence;
          }
        });
      });
      // STEP THRU EACH ITEM ATTRIBUTE
      $.each(item.definingAttributes.keySort('optionSequence'), function(optionKey, option) {
        // DEFINE ATTRIBUTE
        var attribute = config.product.definingAttributes[optionKey];
        var type = attribute.optionType;
        // ADD ITEM SWATCH
        if (type.includes('SWATCHBOX')) {
          item.swatch = option.selectValue;
        }
        // ADD ATTRIBUTE NAME
        option.optionName = attribute.optionName;
        // CHECK IF NESTED ATTRIBUTE EXISTS
        if (!current.nestedAttributes) {
          attribute.optionComponent = type.includes('SELECTIONCHIP') ? 'selection-chip' : type.includes('SWATCHBOX') ? 'swatch-box' : 'list-box';
          // BUILD NESTED ATTRIBUTES
          current.nestedAttributes = $.extend({}, attribute);
          current.nestedAttributes.options = {};
          current.nestedAttributes.optionsLength = 0;
          current.nestedAttributes.optionSequence = optionKey;
          //console.log('New Attribute: ', attribute.optionName);
        }
        // CHECK IF OPTION ITEM KEY MATCHES
        if (current.nestedAttributes.optionItemKey === option.optionItemKey) {
          // CHECK IF CURRENT OPTION EXISTS
          if (!current.nestedAttributes.options[option.displayName]) {
            // BUILD CURRENT OPTIONS
            current.nestedAttributes.options[option.displayName] = $.extend({}, option);
            // ADD SEQUENCE TO OPTION
            current.nestedAttributes.options[option.displayName].displaySequence = current.nestedAttributes.optionsLength++;
            // LAST ATTRIBUTE, SO ADD ITEM
            if (optionKey === last) {
              current.nestedAttributes.options[option.displayName].entitledItemKey = itemKey;
            }
            //console.log('New Option: ', option.displayName);
          }
          if (optionKey === last) {
            // BACK TO FIRST ATTRIBUTE
            current = config.product;
          } else {
            // STEP INTO NEXT ATTRIBUTE
            current = current.nestedAttributes.options[option.displayName];
          }
        } else if (optionKey === last) {
          // BACK TO FIRST ATTRIBUTE
          current = config.product;
        }
      });
    });
  }
  // PRODUCT RENDER
  var productRender = function(){
    // RENDER COMPONENT
    $$.renderTemplate(config);
  }
  // PRODUCT CONTROLLER
  var productController = function(config){
    // DEFINE CONSTANTS
    var $module = config.module;
    var $topper = $module.find('.product-topper');
    var $containers = $module.find('.product-attribute');
    var $messaging = $module.find('.product-messaging');
    var $quantity = $module.find('.product-quantity');
    var $actions = $module.find('.product-actions');
    // BUILD ATTRIBUTE AT LEVEL
    var buildAttributes = function(level){
      var $container = $containers.eq(level-1);
      var $options = $container.find('.options');
      $$.renderTemplate({
        target: $options,
        method: 'replaceWith',
        template: 'product_config',
        render: 'options',
        standalone: config.standalone,
        product: {
          nestedAttributes: getAttributes(level)
        },
        maxAttributes: config.maxAttributes,
        ready: function(){
          handleAttributes(level);
        }
      });
    }
    // GET ATTRIBUTE AT LEVEL
    var getAttributes = function(level){
      var index = 1;
      var current = config.product.nestedAttributes;
      for (index; index < level; index++) {
        current = current.options[config.product.definingAttributes[index-1].optionSelection].nestedAttributes;
      }
      return current;
    }
    // RESET ATTRIBUTES AFTER LEVEL
    var resetAttributes = function(level){
      // RESET CONTAINERS
      $.each($containers, function(index, container){
        if (index < level) {
          return;
        }
        // DEFINE VARIABLE
        $container = $(container);
        // RESET OPTIONS
        if (index > 0) {
          var $options = $container.find('.options');
          setTimeout(function(){
            $options.replaceWith('<span class="options '+config.product.definingAttributes[index].optionComponent+'">Choose '+config.product.definingAttributes[index-1].optionName+' first</span>');
          }, 250);
        }
        // RESET SELECTION
        $container.find('.selection').text('');
        config.product.definingAttributes[index].optionSelection = '';
      });
    }
    // SET FEEDBACK FOR MODULE
    var setFeedback = function(){
      var item = getItem();
      if (item) {
        $quantity.find('.total').html('<span class="title">Total:</span> <span class="price">$'+((item.contractPrice * config.product.quantity).formatCurrency())+'</span>').show();
        if (item.itemAvailableInventory === 0) {
          $messaging.append('<div class="message"><strong>Ships:</strong> '+item.itemAvailabilityDate.replace(/(\d+-\d+-\d+).+/, '$1')+'</div>');
        }
        $.each(item.descriptiveAttributes, function(index, attribute){
          if (attribute.name === 'Shipping Surcharge Message') {
            $messaging.append('<div class="message">'+attribute.value.decodeHTML()+'</div>');
          }
        });
      }
    }
    // RESET FEEDBACK FOR MODULE
    var resetFeedback = function(){
      $quantity.find('.total').html('').hide();
      $messaging.text('');
      $containers.removeClass('in-error');
    }
    // HANDLE OPTION SELECTION
    var handleAttributes = function(level){
      var $container = $containers.eq(level-1);
      var $selection = $container.find('.selection');
      var $options = $container.find('.options');
      var interaction = $options.attr('class').includes('list-box') ? 'change' : 'click';
      var length = config.product.definingAttributes.length;
      // DEFINE CONTROLLER
      if (interaction === 'click') {
        var $controller = $options.find('li').not('.more');
      } else {
        var $controller = $options;
      }
      // HANDLE INTERACTION
      $controller.on(interaction, function(event) {
        var $option = $(this);
        if (interaction === 'click') {
          // SET CLASSES
          $options.find('li').removeClass('selected');
          $option.addClass('selected');
        }
        // GET SELECTED VALUE
        if (interaction === 'click') {
          var value = $option.attr('title');
          var swatch = $option.data('swatch');
        } else if (interaction === 'change') {
          var value = $option.find(':selected').attr('value');
          var swatch = false;
        }
        // SET SELECTION
        $selection.text(value);
        config.product.definingAttributes[level-1].optionSelection = value;
        // SET IMAGE PREVIEW
        if (swatch) {
          var $previews = $module.parents('.'+config.id).find('.product-imagery, .swiper-slide').not('.swiper-slide-duplicate').find('> img[data-src]');
          $.each($previews, function(index, preview){
            var $preview = $(preview);
            var asset = config.product.mfPartNumber+'_'+swatch;
            $$.fetchFeed({
              feed: $$.properties.frameworkPath+'php/scene7_images.php?images='+asset+'&sequential=false&frequency=12',
              callback: function(images){
                if (images[0]) {
                  $preview.attr('src', $$.s7Builder(images[0], $preview.data('src')));
                } else {
                  $preview.attr('src', $preview.data('src'));
                }
                $preview.parent().click();
              }
            });
          });
        }
        // RESET FEEDBACK
        resetFeedback();
        // BUILD ATTRIBUTES
        if (level < length) {
          resetAttributes(level);
          buildAttributes(level+1);
        // ITEM LEVEL STUFF
        } else {
          setFeedback();
        }
      });
      // HANDLE MORE
      $options.find('.more > button').on('click', function(){
        var value = config.product.definingAttributes[level-1].optionSelection;
        $$.renderTemplate({
          template: 'product_config',
          render: 'dialog',
          target: 'body',
          method: 'append',
          content: $(this).parents('.product-attribute').html(true),
          callback: function(config){
            var $container = config.module;
            var $selection = $container.find('.selection');
            var $swatches = $container.find('.options');
            var $controller = $swatches.find('li').not('.more');
            var $confirm = $container.find('.confirm > .btn');
            // HANDLE INTERACTION
            $controller.on('click', function(event) {
              var $swatch = $(this);
              // SET CLASSES
              $swatches.find('li').removeClass('selected');
              $swatch.addClass('selected');
              // GET SELECTED VALUE
              value = $swatch.attr('title');
              // SET SELECTION
              $selection.text(value);
            });
            // HANDLE INTERACTION
            $confirm.on('click', function(event) {
              // SET SELECTION
              $options.find('li').filter('[title="'+value+'"]').click();
            });
            // INITIALIZE MODAL
            $container.modal().on('hidden.bs.modal', function(){
              $(this).remove();
            });
          }
        });
      });
    }
    // GET CONFIGURED ITEM
    var getItem = function(feedback){
      var index = 0;
      var current = config.product;
      var length = config.product.definingAttributes.length;
      var error = false;
      var feedback = typeof feedback !== 'undefined' ? feedback : false;
      for (index; index < length; index++) {
        var selection = config.product.definingAttributes[index].optionSelection;
        if (selection) {
          current = current.nestedAttributes.options[selection];
        } else {
          var error = true;
          if (feedback) {
            $containers.eq(index).addClass('in-error');
          }
        }
      }
      return error ? false : config.product.entitledItems[current.entitledItemKey || 0];
    }
    // RESET HANDLER
    $topper.find('.reset > button').on('click', function(event){
      // RESET QUANTITY
      var quantity = config.product.initialQuantity || 1;
      config.product.quantity = quantity;
      $quantity.find('.quantity > select').val(quantity);
      // RESET IMAGE PREVIEW
      var $previews = $module.parents('.'+config.id).find('.product-imagery, .swiper-slide').not('.swiper-slide-duplicate').find('> img[data-src]');
      $.each($previews, function(index, preview){
        var $preview = $(preview);
        $preview.attr('src', $preview.data('src'));
        $preview.parent().click();
      });
      // RESET FEEDBACK
      resetFeedback();
      // RESET ATTRIBUTES
      resetAttributes(0);
      buildAttributes(1);
    }).trigger('click');
    // QUANTITY HANDLER
    $quantity.find('.quantity > select').on('change', function(event){
      config.product.quantity = parseInt($(this).val());
      resetFeedback();
      setFeedback();
    }).trigger('change');
    // ADD-TO-CART HANDLER
    $actions.find('.add-to-cart > button').on('click', function(event){
      var item = getItem(true);
      if (item) {
        $$.addToCart(item, config.category, config.product.quantity,
          /*
          // OPEN CALLBACK
          function(){
            // HIDE QUICKVIEW
            $$.goModal('hide');
          },
          // CLOSE CALLBACK
          function(){
            // SHOW QUICKVIEW
            $$.goModal('show');
          }
          */
          /*
          // OPEN CALLBACK
          function(){
            // CLOSE QUICKVIEW
            $$.goModal();
          }
          */
          {
            // OPEN CALLBACK
            onOpen: function(){
              // CLOSE QUICKVIEW
              $$.goModal();
            }
          }
        );
      }
    });
  }
  // DEFAULTS
  config = $.extend({
    method: 'append',
    template: 'product_config',
    render: 'module',
    ready: productController
  }, config);
  // ROUTER
  if (config.product) {
    if (typeof config.product === 'object') {
      productModel();
      productRender();
    } else {
      var authKey = "";
      if($$.properties.brandSimple == "frontgate") {
        authKey = 'ed2fac4e-91e9-347f-873b-9ff47cec829f';
      }
      else {
        authKey = 'd62e83a2-a5bf-304e-bf19-c1566104ad29';
      }
      $$.fetchFeed({
        feed: 'https://api.'+$$.properties.brandSimple+'.com/wcs/product/1.0.0/detailByProductId/'+config.product,
        headers: {'Authorization':'Bearer '+authKey},
        //feed: $$.getContent('https://www.'+$$.properties.brandSimple+'.com/JSONAPIs?storeId='+$$.properties.storeId+'&catalogId='+$$.properties.catalogId+'&langId=-1&module=productDetail&productId='+config.product, 12),
        callback: function(response){
          if (response && (response.pageProduct || response.bundle.length > 0)) {
            config.product = response.pageProduct || response.bundle[0].pageProduct;
            productModel();
            productRender();
          } else {
            config.ready = false;
            config.product = false;
            productRender();
          }
        },
        error: function(){
          config.ready = false;
          config.product = false;
          productRender();          
        }
      });
    }
  }
}
/*
$$.buildConfig();
$$.buildConfig({
  product: 766898,
  callback: function(config){
    console.log(config);
  }
});
*/

/*! FUNCTION: $$.BUILDGALLERY(); **/
$$.buildGallery = function(config, callback){
  // SETUP
  if (typeof config === 'undefined') {
    var config = {};
  }
  if (typeof callback === 'function') {
    config.callback = callback;
  }
  // DEFAULTS
  config = $.extend({
    method: 'append',
    template: 'media_gallery',
    ready: function(){
      $window.trigger('swiper-initialize');
    }
  }, config);
  // RENDER
  $$.renderTemplate(config);
}
/*
$$.buildGallery();
$$.buildGallery({
  images: ['b-35398_i1', 'b-35398_i2', 'b-35398_i3', 'b-35398_i4', 'b-35398_i5', 'b-35398_i6', 'b-35398_i7', 'b-35398_i9', 'b-35398_i10'],
  videos: [{
    source: 'http://www.frontgate.com/wcsstore/images/Cinmar/_wcm/html/video.htm?youtube=UPLWaa45F1M&title=10-Ft. Square Side Mount Umbrella',
    thumb: '//frontgate.scene7.com/is/image/frontgate/alt_video?$src=64445_main'
  }],
  paramsSlider: {
    'wid': 736,
    'hei': 360
  }
});
*/

/*! FUNCTION: $$.BUILDPRODUCT(); **/
$$.buildProduct = function(config, callback){
  // SETUP
  if (typeof config === 'undefined' || !config.product) {
    return false;
  }
  if (typeof callback === 'function') {
    config.callback = callback;
  }
  // DEFAULTS
  config = $.extend({
    method: 'append',
    template: 'product_module',
    ready: function(config){
      var bundle = config.product.bundle;
      var products = bundle || (Array.isArray(config.product) ? config.product : [config.product]);
      var leader = products[0];
      var length = products.length;
      var count = 0;
      // BUNDLE
      if (bundle) {
        // ADD TO LENGTH
        length++;
        // GALLERY
        $$.fetchFeed({
          feed: $$.properties.frameworkPath+'php/scene7_images.php?base='+(config.product.bundleMfPartNumber !== 'null' ? config.product.bundleMfPartNumber : config.product.partNumber.replace(/^b(_|-|\s)/, ''))+'&sequential=false&frequency=12',
          callback: function(images){
            $$.buildGallery({
              target: config.module.find('.product-' + config.product.bundleId).find('.product-imagery'),
              method: 'html',
              id: 'product-' + config.product.bundleId,
              standalone: false,
              popup: config.popup,
              images: images.length ? images : undefined,
              videos: leader.pageProduct.descriptiveAttributes && leader.pageProduct.descriptiveAttributes[2] && leader.pageProduct.descriptiveAttributes[2].name === 'Video - 1 - Thumbnail Image Url' ? [{
                source: leader.pageProduct.descriptiveAttributes[4].value,
                thumb: leader.pageProduct.descriptiveAttributes[2].value
              }] : undefined,
              paramsSlider: {
                'wid': 736,
                'hei': 360
              },
              ready: function(config){
                count++;
                if (count >= length) {
                  $window.trigger('swiper-initialize');
                }
              }
            });
          }
        });
      }
      // SINGLES
      $.each(products, function(index, product){
        // DEFINE PRODUCT
        product = product.pageProduct;
        // GALLERY
        $$.fetchFeed({
          feed: $$.properties.frameworkPath+'php/scene7_images.php?base='+product.mfPartNumber+'&sequential=false&frequency=12',
          callback: function(images){
            $$.buildGallery({
              target: config.module.find('.product-' + product.prodId).find('.product-imagery'),
              method: 'html',
              id: 'product-' + product.prodId,
              standalone: false,
              popup: config.popup,
              images: images.length ? images : undefined,
              videos: product.descriptiveAttributes && product.descriptiveAttributes[2] && product.descriptiveAttributes[2].name === 'Video - 1 - Thumbnail Image Url' ? [{
                source: product.descriptiveAttributes[4].value,
                thumb: product.descriptiveAttributes[2].value
              }] : undefined,
              ready: function(config){
                count++;
                if (count >= length) {
                  $window.trigger('swiper-initialize');
                }
              }
            });
          }
        });
        // CONFIG
        $$.buildConfig({
          target: config.module.find('.product-' + product.prodId).find('.product-config'),
          method: 'replaceWith',
          id: 'product-' + product.prodId,
          product: product,
          category: config.category,
          standalone: false
        });
        // TAGGING
        //cmCreateProductviewTag(product.mfPartNumber, product.prodName, $$.properties.siteName, '-_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_-quickview-_--_--_--_--_--_--_--_--_-'+$$.properties.pageId+'-_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_-');
        //console.log(product.mfPartNumber, product.prodName, $$.properties.siteName, '-_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_-quickview-_--_--_--_--_--_--_--_--_-'+$$.properties.pageId+'-_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_--_-');
        $(document).trigger('quickview', [{mfPartNumber: product.mfPartNumber}]);
      });
    }
  }, config);
  // ROUTER
  if (config.product) {
    if (typeof config.product === 'object') {
      $$.renderTemplate(config);
    } else {
      var authKey = "";
      if($$.properties.brandSimple == "frontgate") {
        authKey = 'ed2fac4e-91e9-347f-873b-9ff47cec829f';
      }
      else {
        authKey = 'd62e83a2-a5bf-304e-bf19-c1566104ad29';
      }
      $$.fetchFeed({
        feed: 'https://api.'+$$.properties.brandSimple+'.com/wcs/product/1.0.0/detailByProductId/'+config.product,
        headers: {'Authorization':'Bearer '+authKey},
        //feed: $$.getContent('https://www.'+$$.properties.brandSimple+'.com/JSONAPIs?storeId='+$$.properties.storeId+'&catalogId='+$$.properties.catalogId+'&langId=-1&module=productDetail&productId='+config.product, 12),
        callback: function(response){
          if (response && (response.pageProduct || response.bundle.length > 0)) {
            if(response.pageProduct && response.pageProduct.minimumPrice == 0) {
              config.ready = false;
              config.product = false;
              $$.renderTemplate(config);
            } else {
              config.product = response;
              $$.renderTemplate(config);
            }
          } else {
            config.ready = false;
            config.product = false;
            $$.renderTemplate(config);
          }
        },
        error: function(){
          console.log('Error');
          config.ready = false;
          config.product = false;
          $$.renderTemplate(config);
        }
      });
    }
  }
}
/*
$$.buildProduct();
$$.buildProduct({
  product: 766898,
  callback: function(config){
    console.log(config);
  }
});
*/

/*! FUNCTION: $$.BUILDMODAL(); **/
$$.buildModal = function(config, callbacks){
  /*! CHECK: UNDEFINED? */
  if (typeof config === 'undefined') {
    return false;
  } else
  /*! CHECK: STRING? */
  if (typeof config === 'string') {
    config = {
      content: config
    };
  }
  /*! CHECK: UNDEFINED? */
  if (typeof callbacks === 'undefined') {
    callbacks = {};
  } else
  /*! CHECK: FUNCTION? */
  if (typeof callbacks === 'function') {
    callbacks = {
      onOpen: callbacks
    };
  }
  /*! EXTEND: DEFAULTS */
  config = $.extend({
    className: '',
    title: '',
    content: '',
    onOpen: callbacks.onOpen,
    onClose: callbacks.onClose,
    onOkay: callbacks.onOkay,
    keydown: function(event){
      var key = event.keyCode || event.charCode;
      if (key == 13) {
        $modal.modal('hide');
        $document.off('keydown', config.keydown);
      }
    }
  }, config);
  /*! DEFINE: MODAL */
  var $modal = 
  /*! BUILD: MODAL */
  $('\
  <div class="modal fade'+(config.className ? ' modal--'+config.className : '')+'" tabindex="-1" role="dialog" aria-hidden="true">\
    <div class="modal-dialog">\
      <div class="modal-content">\
        <div class="modal-header">\
          <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>\
          <h3 class="modal-title h4">'+config.title+'</h3>\
        </div>\
        <div class="modal-body">\
          '+config.content+'\
        </div>\
        <div class="modal-footer">\
          <span class="okay"><button class="btn btn-primary" data-dismiss="modal">OK</button></span>\
          '+(config.onOkay ? '<span class="cancel"><button class="btn btn-secondary" data-dismiss="modal">Cancel</button></span>' : '')+'\
        </div>\
      </div>\
    </div>\
  </div>\
  ')
  /*! INITIALIZE: MODAL */
  .modal()
  /*! HANDLE: CLOSE */
  .on('hidden.bs.modal', function(){
    /*! REMOVE: MODAL */
    $(this).remove();
    /*! CALLBACK: CLOSE */
    config.onClose && config.onClose(config);
  });
  /*! CALLBACK: OPEN */
  config.onOpen && config.onClose(config);
  /*! CALLBACK: OKAY */
  config.onOkay && $modal.find('.modal-footer').find('.okay').on('click', function(){
    config.onOkay(config);
  });
  /*! HANDLE: ENTER KEY */
  $document.on('keydown', config.keydown);
}
/*
$$.buildModal('Content');
$$.buildModal({title: 'Title', content: 'Content'});
$$.buildModal({title: 'Title', content: 'Content'}, {onOkay: function(){}});
*/

/*! FUNCTION: BUILDSOCIAL(); **/
$$.buildSocial = function(config){
  if (typeof config !== 'object') {
    config = {};
  }
  config = $.extend({
    theme: '', // 'light', 'dark'
    onclick: '' // 'string'
  }, config);
  var buttons = $.map({
    Pinterest: {
      config: {
        url: config.url,
        description: config.pinterest_description /* || config.houzz_description */ || config.description || config.title,
        image: config.pinterest_image || config.houzz_image || config.image
      },
      title: 'Pin on Pinterest'
    },
    Facebook: {
      config: {
        url: config.url,
        title: config.title,
        description: config.facebook_description || config.description,
        image: config.facebook_image || config.image
      },
      title: 'Share on Facebook'
    },
    Houzz: {
      config: {
        url: config.url,
        description: config.houzz_description /* || config.pinterest_description || config.description */ || config.title,
        image: config.houzz_image || config.pinterest_image || config.image
      },
      title: 'Add to Houzz'
    },
    Twitter: {
      config: {
        url: config.url,
        description: config.twitter_description || config.description || config.title,
        hashtags: config.hashtags
        //image: config.twitter_image || config.image
      },
      title: 'Tweet on Twitter'
    },
    Google: {
      config: {
        url: config.url
      },
      title: 'Share on Google+'
    },
    Email: {
      config: {
        url: config.url,
        title: config.title,
        description: config.email_description || config.description
      },
      title: 'Email a Friend'
    }
  }, function(value, key){
    return '<button class="glyphicon glyphicon-'+key.toLowerCase()+' '+config.theme+'" onclick="$$.share.'+key.toLowerCase()+'('+JSON.stringify(value.config).replace(/"/g, '&quot;')+');'+config.onclick+'" title="'+value.title+'"></button>';
  }).join('');
  return buttons;
}
/*
$$.buildSocial({url: 'http://www.google.com/', title: 'Google', image: 'http://www.google.com/logo.png', description: 'Google is the best!', hash: 'Google'});
*/

/*! FUNCTION: $$.BUILDVIDEO(); **/
$$.buildVideo = function(options){
  var container;
  var player;
  var title;
  var status;
  var current;
  var duration;
  
  options = $.extend({
    target: '.video',
    method: 'appendTo',
    toggle: false,
    destroy: true,
    onInit: function(){
      if (options.toggle){
        $(options.toggle).toggle();
      }
      container = $('<div>').addClass('aspect').append(
        $('<div>').attr('id', 'player-'+options.src)
      )[options.method]($(options.target));
    },
    onReady: function(event){
      if (!$$.tests.touch) {
        player.playVideo();
      }
      title = document.title.replace(/(\s+\|\s+)?Home\s*\+\s*Style/, '');
      duration = Math.round(player.getDuration());
    },
    onChange: function(event){
      switch (event.data){
        case 0: status = 3; break; // ENDED
        case 1: status = 2; break; // PLAYING
        case 2: status = 1; break; // PAUSED
        default: status = 0; break; // STARTED ???
      }
      
      current = Math.round(player.getCurrentTime());
      window.cmCreateElementTag && cmCreateElementTag('Video: '+title+'', $$.properties.siteName, '-_--_--_--_--_--_--_--_--_--_--_--_-'+status+'-_-'+current+'-_-'+duration+'');
      //console.log('Video: '+title+'', $$.properties.siteName, '-_--_--_--_--_--_--_--_--_--_--_--_-'+status+'-_-'+current+'-_-'+duration+'');
      
      if (event.data == 0){
        if (options.toggle){
          $(options.toggle).toggle();
          $$.forceRedraw();
        }
        if (options.destroy){
          player.destroy();
          container.remove();
        }
        options.onEnd();
      }
    },
    onEnd: $.noop
  }, options);
  
  options.src = options.src.replace('.be','be.com').replace(/^((https?:)?\/\/)?(www\.)?/,'').replace(/(youtube|vimeo)\.com\/(watch\?v=)?/,'').replace(/\?.*$/,'');
  
  onYouTubeIframeAPIReady = function(){
    player = new YT.Player('player-'+options.src, {
      height: '100%',
      width: '100%',
      videoId: options.src,
      playerVars: {
        autoplay: 1,
        rel: 0
      },
      events: {
        'onReady': options.onReady,
        'onStateChange': options.onChange
      }
    });
    delete window.onYouTubeIframeAPIReady;
  }
  
  options.onInit();
  
  if (window.YT){
    onYouTubeIframeAPIReady();
  } else {
    (function(a, b){
      a = document.createElement('script');
      a.src = '//www.youtube.com/iframe_api';
      b = document.getElementsByTagName('script')[0];
      b.parentNode.insertBefore(a, b);
    })();
  }
}
/*
$$.buildVideo({src: 'http://youtu.be/SPfr9GuEHb8?autoplay=1&rel=0', target: '.section-hero .video', toggle: '.section-hero .hero'});
*/

/*! FUNCTION: $$.ENCODEQUERYSTRING() **/
$$.encodeQueryString = function(options, url){
  var options = (typeof options != 'object') ? $$.parseQueryString(options) : options;
  var url = (typeof url != 'undefined') ? url : '';
  var array = [];
  for (var option in options){
    array.push(encodeURIComponent(option).replace(/%24/g, '$') + "=" + encodeURIComponent(options[option]));
  }
  return (url + (url.indexOf('?') > -1 ? '&' : '?') + array.join('&')).replace(/[?&]+$/,'');
}
/*
$$.encodeQueryString('?new=1&test=true'); // '?new=1&test=true'
$$.encodeQueryString('?new=1&test=true', 'google.com'); // 'google.com?new=1&test=true'
$$.encodeQueryString('?new=1', 'google.com?old=0'); // 'google.com?old=0&new=1'
$$.encodeQueryString('test.com?new=1', 'google.com?old=0'); // 'google.com?old=0&new=1'
$$.encodeQueryString({new: 1, test: true}); // '?new=1&test=true'
$$.encodeQueryString({new: 1, test: true}, 'google.com'); // 'google.com?new=1&test=true'
$$.encodeQueryString({new: 1}, 'google.com?old=0'); // 'google.com?old=0&new=1'
*/

/*! FUNCTION: $$.PARSEQUERYSTRING() **/
$$.parseQueryString = function(query, normalize){
  var query = ((typeof query != 'undefined') ? query : window.location.href).replace(/.+?([#?])/, '$1');
  var object = {};
  query.replace(/[?&#]*([^=&]+)=([^&#]*)/g, function(match, key, value){
    value = decodeURIComponent(value.replace(/\+/g, ' '));
    object[key] = 
      (typeof normalize != 'undefined' && normalize == true) ? 
      (value) : 
      (/^(true|false)$/.test(value)) ? 
      (value == 'true') : 
      (!isNaN(value.replace(/^\s*$/, false))) ? 
      (parseFloat(value)) : 
      (value);
  });
  return object; 
}
/*
$$.parseQueryString(); // Parameters From Current URL
$$.parseQueryString()['test']; // Value Of "test" From Current URL
$$.parseQueryString('?width=800&height=600'); // {width: 800, height: 600}
$$.parseQueryString('noparam?width=800&height=600'); // {width: 800, height: 600}
$$.parseQueryString('//frontgate.scene7.com/is/image/frontgate/Collections_Hero_Orleans?$wfsc$');
*/

/*! FUNCTION: $$.FLEXBOX(); **/
$$.flexBox = function($flex, $static, callback){
  if (
    typeof $flex != 'undefined' && $flex.length > 0 &&
    typeof $static != 'undefined' && $static.length > 0
  ) {
    /*! DEFINE: VARIABLES */
    var $container = $static.parent().eq(0);
    /*! HANDLE: THROTTLED RESIZE */
    $window.on('resize',
      $.throttle(10, function(event){
        if (event.target === window) {
          /*! DEFINE: VARIABLES */
          var container = $container.outerWidth();
          var static = $static.outerWidth();
          var width = (container - static) || '';
          /*! SET: DIMENSIONS */
          $flex.css('width', width);
          /*! TRIGGER: CALLBACK */
          typeof callback == 'function' && callback.call($flex, event);
        }
      })
    );
    /*! RETURN: TRUE */
    return true;
  }
  /*! RETURN: FALSE */
  return false;
}
/*
$$.flexBox(
  $('.flex-dynamic'),
  $('.flex-static'),
  function(event){
    console.log('Flexbox');
  }
);
*/

/*! FUNCTION: $$.FETCHFEED() **/
$$.fetchFeed = function(config){
  if (typeof config === 'undefined' || (!config.feed && !config.feed1)){
    console.error('Error: Missing Data!');
    return false;
  }
  
  config = $.extend({
    type: typeof config.callback === 'string' ? 'jsonp' : 'json',
    error: function(xhr, status, error){
      console.error('Error: '+error);
    },
    callback: function(output){
      console.dir(output);
    }
  }, config);
  
  var output = {};
  
  function parseCsv(data){
    $.csv.toObjects(data, {}, function(error, obj){
      data = obj;
    });
    return data;
  }

  (function loopAjax(index){
    if (config['feed'+index]){
      $.ajax({
        url: config['feed'+index],
        dataType: (config['type'+index] === 'csv' ? 'text' : config['type'+index] || config['type']),
        cache: true,
        headers: config['headers'] || {'Authorization':'Bearer '+ $$.properties.apiKey},
        jsonpCallback: typeof config.callback === 'string' ? config.callback : false,
        error: config.error,
        success: function(data){
          config['type'+index] === 'csv' && (output['data'+index] = parseCsv(data)) || (output['data'+index] = data);
        },
        complete: function(xhr, status){
          loopAjax(index+1);
        }
      });
    } else {
      var data = !output.data2 ? (output.data1 || output.data) : output;
      if (typeof config.callback === 'function' && data) {
        config.callback(data);
      }
    }
  }(config['feed'] ? '' : 1));

  return true;
}
/*
$$.fetchFeed({
  feed1: '/JSONAPIs?storeId='+$$.properties.storeId+'&catalogId='+$$.properties.catalogId+'&langId=-1&module=categoryDetail&categoryId=83155',
  feed2: '/wcsstore/images/Frontgate/_media/Landing/master_suite/feed/data.csv', type2: 'csv'
});
*/

/*! FUNCTION: $$.GETCONTENT(); **/
$$.getContent = function(url, frequency){
  var frequency = (typeof frequency !== 'undefined') ? frequency : 1;
  var proxy = $$.properties.frameworkPath+'php/get_content.php';
  return (typeof url !== 'undefined' ? $$.encodeQueryString({url: url, frequency: frequency}, proxy) : proxy);
}
/*
getContent('http://www.frontgate.com/JSONAPIs?module=productGrid&categoryId=95852');
*/

/*! FUNCTION: $$.PROXYURL(); **/
$$.proxyUrl = function(url, frequency){
  return $$.getContent.apply(this, arguments);
}
/*
proxyUrl('http://www.frontgate.com/JSONAPIs?module=productGrid&categoryId=95852');
*/

/*! FUNCTION: $$.FORCEREDRAW() **/
$$.forceRedraw = function(){
  window.scrollTo(0,window.pageYOffset+1);
  window.scrollTo(0,window.pageYOffset-1);
  if (document.createEvent){
    var resize = document.createEvent('HTMLEvents');
    resize.initEvent('resize', true, false);
    document.body.dispatchEvent(resize);
  } else if (document.createEventObject){
    document.body.fireEvent('onresize');
  } else {
    $(window).trigger('resize');
  }
};
/*
$$.forceRedraw();
*/

/*! FUNCTION: $$.GETTEMPLATE(); **/
$$.getTemplate = (function(){
  var templates = {};
  return function(config, callback){
    // SETUP
    if (typeof config === 'undefined') {
      return false;
    } else 
    if (typeof config === 'string') {
      var template = config; 
      config = {};
      config.template = template;
    }
    if (typeof callback === 'function') {
      config.callback = callback;
    }
    // DEFAULTS
    config = $.extend({
      path: $$.properties.templatePath || $$.properties.wordpressPath+'wp-framework/html/',
      prefix: 'template_',
      suffix: '',
      extension: '.htm',
      query: $$.properties.wordpressHash ? '?ver='+$$.properties.wordpressHash : '',
      template: undefined,
      callback: undefined
    }, config);
    // FETCH
    if (templates[config.template]) {
      config.callback && config.callback(templates[config.template]);      
    } else {
      $$.fetchFeed({
        type: 'text',
        feed: config.path + config.prefix + config.template + config.suffix + config.extension + config.query,
        callback: function(response){
          templates[config.template] = response;
          config.callback && config.callback(response);
        }
      });
    }
  }
})();
/*
$$.getTemplate('media_gallery', function(template){
  console.log(template);
});
$$.getTemplate({
  template: 'media_gallery',
  path: $$.properties.wordpressPath+'wp-framework/html/',
  callback: function(template){
    console.log(template);
  }
});
*/

/*! FUNCTION: $$.RENDERTEMPLATE(); **/
$$.renderTemplate = (function(){
  return function(config, callback){
    // SETUP
    if (typeof config === 'undefined') {
      return false;
    } else 
    if (typeof config === 'string') {
      var template = config; 
      config = {};
      config.template = template;
    }
    if (typeof callback === 'function') {
      config.callback = callback;
    }
    // DEFAULTS
    config = $.extend({
      target: 'main',
      method: undefined,
      template: undefined,
      ready: undefined,
      callback: undefined
    }, config);
    // CONFIG
    var settings = $.extend({}, config);
    delete settings.ready;
    delete settings.callback;
    // TEMPLATE
    $$.getTemplate(settings, function(template){
      var complete = function(){
        config.html = _.template(template, {variable: 'config'})(config);
        config.module = $(config.html);
        if (config.method) {
          $(config.target)[config.method](config.module).promise().done(callbacks);
        } else {
          callbacks();
        }
      }
      var callbacks = function(){
        config.ready && config.ready(config);
        config.callback && config.callback(config);
      }
      if (window._) {
        complete();
      } else {
        $.getScript(($$.properties.frameworkPath || $$.properties.wordpressPath+'wp-framework/')+'js/libraries/underscore.js', complete);
      }
    });
  }
})();
/*
$$.renderTemplate('media_gallery', function(config){
  console.log(config);
});
$$.renderTemplate({
  template: 'media_gallery',
  method: 'append',
  callback: function(config){
    console.log(config);
  }
});
*/

/*! FUNCTION: $$.GOMODAL(); **/
$$.goModal = function(options){
  // PARSE PARAMETERS
  if (typeof options === 'undefined'){
    $.fancybox.close();
    return;
  } else if (typeof options === 'string'){
    if (options === 'show' || options === 'hide') {
      $.fancybox.isOpen && $.fancybox.current.locked['fade'+(options === 'show' ? 'In' : 'Out')]();
      return;
    } else if (/^(https|\/)/.test(options)) {
      var options = {
        href: options
      }
    } else {
      var options = {
        content: options
      }
    }
  }
  // FIX FANCYBOX-LOCK
  if ($.fancybox.isOpen) {
    var afterLoad = options.afterLoad || $.noop;
    options.afterLoad = function(){
      $('html').addClass('fancybox-lock');
      afterLoad();
    }
  }
  // SET CALLBACKS
  if (options.callbacks) {
    if (typeof options.callbacks == 'function') {
      options.callbacks = [options.callbacks];
    }
    options.afterShow = function(){
      $.each(options.callbacks, function (index, callback) {
        callback();
      });
    }
  }
  // FROM DOM ELEMENT
  if (options.element) {
    var href = options.element.href;
    var title = options.element.title || options.element.alt;
    if (!options.href && href) {
      options.href = href;
    }
    if (!options.title && title) {
      options.title = title;
    }
  }
  // DEFINE DEFAULTS
  var defaults = {
    type: 'iframe'
  }
  // MERGE OPTIONS
  options = $.extend({}, defaults, options);
  // LEGACY SUPPORT
  if (options.html) {
    options.content = options.html;
  }
  // INITIALIZE MODAL
  $.fancybox(options);
  // RETURN FALSE
  return false;
}
/*
$$.goModal('//www.frontgate.com/wcsstore/images/GrandinRoad/_media/Landing/HalloweenHaven2013/category-disclaimer.htm');
$$.goModal('<h1>Test H1 Headline</h1><h2>Test H2 Sub-Headline</h2><p>Test Paragraph</p>');
$$.goModal({href: '//www.frontgate.com/wcsstore/images/GrandinRoad/_media/Landing/HalloweenHaven2013/category-disclaimer.htm', width: 600});
$$.goModal({content: '<h1>Test H1 Headline</h1><h2>Test H2 Sub-Headline</h2><p>Test Paragraph</p>', width: 600});
*/

/*! FUNCTION: $$.QUICKCART(); **/
$$.quickCart = function(callbacks){
  if (typeof callbacks === 'undefined') {
    var callbacks = {};
  }
  // SHOW LOADING
  $.fancybox.showLoading();
  // CLOSE ON MOBILE
  var updateBreakpoints = function(event, data){
    if (data.key === 'xs' && data.value < $$.breakpoints.previous) {
      $$.goModal();
    }
  }
  // CONFIGURE MODAL
  var targetWidth = 800-30;
  var targetheight = 600-30;
  $$.goModal({
    parent: '#popouts',
    wrapCSS: 'fancybox-cart',
    type: 'inline',
    href: '#cart',
    margin: [0, 0, 0, 0],
    width: targetWidth,
    maxWidth: targetWidth,
    height: targetheight,
    maxHeight: targetheight,
    autoSize: false,
    scrolling: 'no',
    topRatio: 0,
    leftRatio: 1,
    beforeShow: function(){
      callbacks.beforeOpen && callbacks.beforeOpen();
    },
    afterShow: function(){
      $window.on('breakpoint', updateBreakpoints);
      callbacks.onOpen && callbacks.onOpen();
    },
    beforeClose: function(){
      callbacks.beforeClose && callbacks.beforeClose();
    },
    afterClose: function(){
      $window.off('breakpoint', updateBreakpoints);
      callbacks.onClose && callbacks.onClose();
    }
  });
}
/*
$$.quickCart();
$$.quickCart(callbacks);
*/

/*! FUNCTION: $$.QUICKVIEW(); **/
$$.quickView = function(product, category, callbacks){
  if (typeof product === 'undefined') {
    return false;
  }
  /*
  if (typeof category === 'undefined') {
    var category = $$.properties.siteName;
  }
  */
  if (typeof callbacks === 'undefined') {
    var callbacks = {};
  }
  /*
  if (!target) {
    $.fancybox.showLoading();
  }
  */
  // SHOW LOADING
  $.fancybox.showLoading();
  // BUILD PRODUCT
  $$.buildProduct({
    target: /* target */ false,
    product: product,
    category: category,
    popup: true,
    callback: /* target ? $.noop : */ function(config){
      var updateSettings = function(event, data){
        if (/*data.key === 'xs' && data.value < $$.breakpoints.previous*/ $$.breakpoints.current === 0) {
          $.extend($.fancybox.current, smallSettings);
        } else if (/*data.key !== 'xs' && data.value > $$.breakpoints.previous*/ $$.breakpoints.previous === 0) {
          $.extend($.fancybox.current, mediumSettings);
        }
      }
      var smallScreen = $$.breakpoints.get().key === 'xs';
      var smallSettings = {
        topRatio: 1,
        openEffect: 'elastic',
        closeEffect: 'elastic'
      }
      var mediumSettings = {
        topRatio: 0.5,
        openEffect: 'fade',
        closeEffect: 'fade'
      }
      $$.goModal($.extend({
        parent: 'body',
        wrapCSS: 'fancybox-auto-height product-quickview',
        content: config.module,
        margin: [0, 0, 0, 0],
        maxWidth: (950-30),
        height: 'auto',
        scrolling: 'no',
        afterShow: function(){
          $window.on('breakpoint', updateSettings);
          $window.trigger('swiper-initialize');
          callbacks.onOpen && callbacks.onOpen(product, category);
          dataLayer.push({'event':'quickview-open'});
        },
        afterClose: function(){
          $window.off('breakpoint', updateSettings);
          $window.trigger('swiper-destroy');
          callbacks.onClose && callbacks.onClose(product, category);
        }
      }, smallScreen ? smallSettings : mediumSettings));
    }
  });
}
/*
$$.quickView(771333);
$$.quickView(631657);
$$.quickView(787882, 95704);
*/

/*! FUNCTION: $$.S7ASSET(); **/
$$.s7Asset = function(url){
  return url.replace(new RegExp('.+?\/('+$$.properties.s7Render+'|'+$$.properties.s7Server+')\/(T_With(out)?Zoom.+?\$src=)?'), '').replace(/^\//, '').replace(/^([^\?\&\/]+).*/, '$1');
}
/*
$$.s7Asset('https://s7ondemand1.scene7.com/is/image/frontgate/assetName'); // 'assetName'
$$.s7Asset('https://a248.e.akamai.net/f/248/6062/6h/s7ondemand1.scene7.com/is/image/frontgate/assetName?param=value&src={nestedImage?param=value}'); // 'assetName'
*/

/*! FUNCTION: $$.S7BUILDER(); **/
$$.s7Builder = function(asset, params){
  if (typeof params === 'undefined') {
    var params = {};
  } else if (typeof params === 'string') {
    params = $$.parseQueryString(params);
  }
  return $$.s7Urlizer(
    $$.s7Asset(asset)+$$.encodeQueryString(
        $.extend(
          $$.parseQueryString(asset),
          params
        )
    ).replace(/([?&#])width=/,'$1wid=').replace(/([?&#])height=/,'$1hei=').replace(/([?&#])quality=/,'qlt=').replace(/([?&#])mode=/,'$1resMode=').replace(/([?&#])sharpen=/,'$1op_usm=')
  );
}
/*
$$.s7Builder('assetName'); // '//frontgate.scene7.com/is/image/frontgate/assetName'
$$.s7Builder('assetName?param=test'); // '//frontgate.scene7.com/is/image/frontgate/assetName?param=test'
$$.s7Builder('assetName',{'key':'value'}); // '//frontgate.scene7.com/is/image/frontgate/assetName?key=value'
$$.s7Builder('assetName?param=test',{'key':'value'}); // '//frontgate.scene7.com/is/image/frontgate/assetName?param=test&key=value'
*/

/*! FUNCTION: $$.S7PRESET(); **/
$$.s7Preset = function(url){
  return url.replace(/^[^=&$]+/, '').replace(/[^=&$]+$/, '').replace(/^\$$/, '');
}
/*
$$.s7Preset('//frontgate.scene7.com/is/image/frontgate/Collections_Hero_Orleans?$wfsc$'); // '$wfsc$'
*/

/*! FUNCTION: $$.S7URLIZER(); **/
$$.s7Urlizer = function(asset, render){
  return '//'+$$.properties.brandSimple+'.scene7.com/is/image/'+(typeof render != 'undefined' && render == true ? $$.properties.s7Render : $$.properties.s7Server)+'/'+asset;
}
/*
$$.s7Urlizer('assetName'); // '//frontgate.scene7.com/is/image/frontgate/assetName'
$$.s7Urlizer('assetName', true); // '//frontgate.scene7.com/is/image/frontgateRender/assetName'
*/

/*! FUNCTION: $$.SHARE(); **/
$$.share = (function(){
  var network,
      location = document.location,
      encode = encodeURIComponent,
      subdomain = /(dev|stage|qa)(wcs)?/,
      build = function(url){
        return url.indexOf('https') === 0 ? 
                 url.replace(subdomain, 'www') : 
               url.indexOf('//') === 0 ? 
                 'https:' + url.replace(subdomain, 'www') : 
               url.indexOf('/') === 0 ? 
                 location.protocol + '//' + location.host.replace(subdomain, 'www') + url : 
               '';
      },
      infer = function(config){
        var $empty = $({content: '', data: ''}),
            $meta,
            data,
            filters = {
              share: function(){
                return config.share === false ? false : true;
              },
              popup: function(){
                if (network === 'Email') {
                  return false;
                }
                return config.popup === false ? false : true;
              },
              auto: function(){
                return config.auto === false ? false : true;
              },
              url: function(){
                $meta = $('meta[property="og:url"]').exists() || $empty;
                return build($meta.prop('content') || location.href);
              },
              title: function(){
                $meta = $('meta[property="og:title"]').exists() || $empty;
                return $meta.prop('content') || document.title;
              },
              description: function(){
                $meta = $('meta[property="og:description"]').exists() || $('meta[name="description"]').exists() || $empty;
                return $meta.data()[network.toLowerCase()] || (network === 'Houzz' ? filters.title() : $meta.prop('content'));
              },
              image: function(){
                $meta = $('meta[property="og:image"]').exists() || $empty;
                return build($meta.data()[network.toLowerCase()] || $meta.prop('content'));
              },
              app_id: function(){
                $meta = $('meta[property="fb:app_id"]').exists() || $empty;
                return $meta.prop('content') || ($$.properties.brandName === 'Frontgate' ? 364305560383973 : 1069039313146835);
              },
              hashtags: function(){
                $meta = $('meta[name="twitter:hashtags"]').exists() || $empty;
                return $meta.prop('content');
              }
            };
        if (typeof config === 'undefined') {
          config = {};
        }
        $.each(filters, function(asset, filter){
          data = config[asset];
          if (typeof data === 'string') {
            if (asset === 'image' || asset === 'url') {
              config[asset] = build(data);
            }
          } else {
            config[asset] = filter();
          }
        });
        return config;
      },
      popup = function(config){
        config = $.extend({
          href: 'about:blank',
          width: 640,
          height: 440
        }, config);
        config.top = (window.screen.height / 2) - ((config.height / 2) + 50);
        config.left = (window.screen.width / 2) - ((config.width / 2) + 10);
        return window.open(config.href, 'sharer', 'menubar=0, toolbar=0, status=0, resizable=1, scrollbars=1, width='+config.width+', height='+config.height+', left='+config.left+', top='+config.top+', screenX='+config.left+', screenY='+config.top);
      },
      share = function(config){
        if (config.share) {
          var follow = function(){
            try {
              if (!config.popup) {
                throw false;
              }
              popup(config);
            } catch (error) {
              location.href = config.href;
            }
            track(config);
          };
          if (/Firefox/.test(navigator.userAgent)) {
            setTimeout(follow, 0);
          } else {
            follow();
          }
        }
      },
      track = function(config){
        config = $.extend({
          // url: ''
          title: ''
          // image: ''
        }, config);
        // TRACK COREMETRICS
        window.cmCreateElementTag && cmCreateElementTag(('Share - '+network+': '+config.title).substring(0, 48), $$.properties.siteName);
        // TRACK WORDPRESS
        setTimeout(function(){
          var $form = $body.find('#meta');
          if ($form.length > 0){
            $.ajax({
              type: $form.attr('method'),
              url: $form.attr('action'),
              data: $form.serialize()+'&action=add_social&share_type='+network.toLowerCase().replace('+','')
            });
          }
        }, 0);
      }
  return {
    utilities: {
      build: build,
      infer: infer,
      popup: popup,
      share: share,
      track: track
    },
    element: function(element, network){
      var $wrapper = $(element).parents('.wrapper').eq(0);
      if ($wrapper.exists()) {
        var network = network.toLowerCase();
        var config = {
              url: ($wrapper.find('.btn-primary[href]').exists() || $wrapper.find('.btn-secondary[href]').exists() || $wrapper.find('.btn-tertiary[href]').exists() || $wrapper.find('a[href]').exists() || $('<div>').attr('href', '/')).attr('href'),
              title: ($wrapper.find('.title').exists() || $wrapper.find('hgroup, h1, h2, h3, h4, h5, h6').eq(0)).text().deSpace(' '),
              description: ($wrapper.find('.description > *').exists() || $wrapper.find('p').eq(0)).contents().map(function() {return $(this).text()}).get().join(' - ').deSpace(' '),
              image: ($wrapper.find('.image[src]').exists() || $wrapper.find('img[src]')).attr('src')
            }
        return $$.share[network](config);
      }
      return false;
    },
    pinterest: function(config){
      dataLayer.push({"event":"pinterestShare"});
      network = 'Pinterest';
      config = infer(config);
      if(!config.image) {
        config.image = $('meta[property="og:image"]').attr('content');
      }
      // if (!config.auto || !config.image){
      //   (function(a,b){a=document.createElement('script');a.src='//assets.pinterest.com/js/pinmarklet.js';b=document.getElementsByTagName('script')[0];b.parentNode.insertBefore(a,b);})();
      //   track(config);
      // } else {
        config = $.extend({
          href: 'https://pinterest.com/pin/create/button/?url='+encode(config.url)+'&media='+encode(config.image)+'&description='+encode(config.description),
          width: 770,
          height: 626
        }, config);
        share(config);
      // }
      // track(config);
      return config;
    },
    houzz: function(config){
      dataLayer.push({"event":"houzzShare"});
      network = 'Houzz';
      config = infer(config);
      // if (!config.auto || !config.image){
      //   (function(a,b){a=document.createElement('script');a.src='//www.houzz.com/js/clipperBookmarklet.js';b=document.getElementsByTagName('script')[0];b.parentNode.insertBefore(a,b);})();
      //   track(config);
      // } else {
        config = $.extend({
          href: 'https://www.houzz.com/imageClipperUpload?imageUrl='+encode(config.image)+'&title='+encode(config.description)+'&link='+encode(config.url),
          width: 900,
          height: 480
        }, config);
        share(config);
      // }
      // track(config);
      return config;
    },
    facebook: function(config){
      dataLayer.push({"event":"facebookShare"});
      network = 'Facebook';
      config = infer(config);
      config = $.extend({
        href: (function(){
          if (!config.auto || !config.image){
            return 'https://www.facebook.com/sharer/sharer.php?u='+encode(config.url);
          } else {
            return 'https://www.facebook.com/dialog/feed?_path=feed&app_id='+config.app_id+'&client_id='+config.app_id+'&redirect_uri='+encode('https://www.'+$$.properties.brandSimple+'.com/wcsstore/images/Cinmar/_wcm/html/close.htm')+'&display=popup&link='+encode(config.url)+'&caption='+$$.properties.brandSimple+'.com&name='+encode(config.title)+'&description='+encode(config.description)+'&picture='+encode(config.image);
          }
        }()),
        width: 600,
        height: 400
      }, config);
      share(config);
      // track(config);
      return config;
    },
    twitter: function(config){
      dataLayer.push({"event":"twitterShare"});
      network = 'Twitter';
      config = infer(config);
      config = $.extend({
        href: 'https://twitter.com/intent/tweet?text='+encode(config.description)+'&url='+encode(config.url)+'&hashtags='+encode(config.hashtags)+'&via='+$$.properties.brandName.replace(/\s+/, ''),
        width: 600,
        height: 432
      }, config);
      share(config);
      // track(config);
      return config;
    },
    google: function(config){
      dataLayer.push({"event":"googleShare"});
      network = 'Google';
      config = infer(config);
      config = $.extend({
        href: 'https://plus.google.com/share?url='+encode(config.url),
        width: 495,
        height: 495
      }, config);
      share(config);
      // track(config);
      return config;
    },
    email: function(config){
      dataLayer.push({"event":"emailShare"});
      network = 'Email';
      config = infer(config);
      config = $.extend({
        href: 'mailto:?subject='+encode(config.title)+'&body='+encode((config.description ? config.description+'\n\n' : '')+config.url)
      }, config);
      share(config);
      // track(config);
      return config;
    }
  }
})();
/*
$$.share.pinterest();
$$.share.houzz();
$$.share.facebook();
$$.share.twitter();
$$.share.google();
*/

/*! TRIGGER: FUNCTIONS **/
$(window).trigger('functions');
/*! LOG: STATUS **/
console.info('Event - Functions Loaded');